/*

Based on code containing the following notice:

Copyright _ 2005, Apple Computer, Inc.  All rights reserved.
NOTE:  Use of this source code is subject to the terms of the Software
License Agreement for Mac OS X, which accompanies the code.  Your use
of this source code signifies your agreement to such license terms and
conditions.  Except as expressly granted in the Software License Agreement
for Mac OS X, no other copyright, patent, or other intellectual property
license or right is granted, either expressly or by implication, by Apple.

And heavily modified by Christopher Masto <chris@masto.com>.  A few
lines, mostly the timing, remain.

*/ 

/*
 *******************************************************************************
 * <Animator object definition.  Smoothly animates from one value to another.> *
 *******************************************************************************
 */
 

/*
 * Animator constructor.  Parameters:
 *
 * -- callBack: Function that gets called for each tick
 * -- startPosition: Initial state of animation variable
 *
 */
function Animator (callBack, startPosition) {
    this.positionNow  = startPosition;
	this.callBack     = callBack;

	this.startTime = 0;
	this.timer = null;
	
	this.animating = false;
	
	this.animate = Animator_animate;
	this.animateTo = Animator_animateTo;
	this.tick = Animator_tick;
}

/*
 * This should only be called via an Animator instance, i.e. "instance.animate(event)"
 * Calling Animator_animate() directly will result in "this" evaluating to the window
 * object, and the function will fail.  Parameters:
 * 
 * -- positionFrom: Where to begin
 * -- positionTo: Where to end up
 * -- animateTime: Over how long to perform the animation
 * -- doneNotification: A callback (if no callback is needed, pass null)
 */
function Animator_animate (positionFrom, positionTo, animateTime, doneNotification) {
	if (this.animating) return;
	
	this.positionFrom      = positionFrom;
	this.positionTo        = positionTo;
	this.animateTime = animateTime;
	this.doneNotification  = doneNotification;

	var timeNow = (new Date).getTime();
	this.startTime = timeNow - 13; // set it back one frame.
	
	// We need to store this in a local variable so the timer does not lose scope
	// when invoking tick.
	var localThis = this;
	this.animating = true;
	this.timer = setInterval (function() { localThis.tick(); }, 13);
	this.tick();
}

// Just like animate(), but start from the current position
function Animator_animateTo(positionTo, animateTime, doneNotification) {
    this.animate(this.positionNow, positionTo, animateTime, doneNotification);
}
		
/*
 * Tick does all the incremental work.
 * This function is very similar to the tick() function in the Fader sample.
 */
function Animator_tick () {
	var T;
	var ease;
	var time  = (new Date).getTime();
	var currentVal;
		
	T = limit_3(time-this.startTime, 0, this.animateTime);
	ease = 0.5 - (0.5 * Math.cos(Math.PI * T / this.animateTime));

	if (T >= this.animateTime) {
		currentVal = this.positionTo;
		clearInterval (this.timer);
		this.timer = null;
		this.positionNow = parseInt(currentVal);
		this.callBack(currentVal);
		if (this.doneNotification) {
			// call after the last frame is drawn
			var localThis = this;
			setTimeout (function() { localThis.doneNotification(); }, 0);
		}
		this.animating = false;
	} else {
		currentVal = computeNextFloat(this.positionFrom, this.positionTo, ease);
		// convert to a integer, not sure if this is the best way
		this.positionNow = parseInt(currentVal);
		this.callBack(currentVal);
	}
}

/*
 * Support functions for the animation
 */
function limit_3 (a, b, c) {
    return a < b ? b : (a > c ? c : a);
}

function computeNextFloat (from, to, ease) {
    return from + (to - from) * ease;
}